@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2020
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -28,58 +29,66 @@ namespace polymake { namespace polytope {
 
 
 template <typename Scalar>
-perl::Object regularity_lp(const Matrix<Scalar> &verts,const Array<Set <int> >& subdiv,perl::OptionSet options) {
-   
-   if (subdiv.size()<2)
+BigObject regularity_lp(const Matrix<Scalar> &verts, const Array<Set<Int>>& subdiv, OptionSet options)
+{
+   if (subdiv.size() < 2)
       throw std::runtime_error("Subdivision is trivial.");   
 
-   std::pair<const Matrix<Scalar>,const Matrix<Scalar> > mats = secondary_cone_ineq(verts,subdiv,options);
-   const Matrix<Scalar>& inequs = mats.first;
-   const Matrix<Scalar>& equats = mats.second;
+   const auto mats = secondary_cone_ineq(full_dim_projection(verts), subdiv, options);
+   const auto& inequs = mats.first;
+   const auto& equats = mats.second;
    
-   const int n_vertices=verts.rows();
+   const Int n_vertices = verts.rows();
 
    Scalar epsilon = options["epsilon"];
-   perl::Object q(perl::ObjectType::construct<Scalar>("Polytope"));
+   BigObject q("Polytope", mlist<Scalar>());
    q.take("FEASIBLE") << true;
    if (equats.rows() > 0)
-     q.take("EQUATIONS") << (zero_vector<Scalar>() | equats);
+      q.take("EQUATIONS") << (zero_vector<Scalar>() | equats);
    q.take("INEQUALITIES") << 
       (zero_vector<Scalar>(n_vertices) | unit_matrix<Scalar>(n_vertices)) /
       ((-epsilon * ones_vector<Scalar>(inequs.rows())) | inequs);
 
-   perl::Object lp(perl::ObjectType::construct<Scalar>("LinearProgram"));
+   BigObject lp = q.add("LP", "LINEAR_OBJECTIVE", 0 | ones_vector<Scalar>(n_vertices));
    lp.attach("INTEGER_VARIABLES") << Array<bool>(n_vertices, true);
-   lp.take("LINEAR_OBJECTIVE") << (Scalar(0) | ones_vector<Scalar>(n_vertices));
-   q.take("LP") << lp;
-   
    return q;
 }
 
 
 
 template <typename Scalar>
-std::pair<bool,Vector<Scalar> > is_regular(const Matrix<Scalar> &verts,const Array<Set <int> >& subdiv,perl::OptionSet options)
+std::pair<bool,Vector<Scalar>>
+is_regular(const Matrix<Scalar> &verts, const Array<Set<Int>>& subdiv, OptionSet options)
 {
-   std::pair<const Matrix<Scalar>, const Matrix<Scalar> > mats = secondary_cone_ineq<Scalar, Set<int> >(verts,subdiv,options);
+   const auto mats = secondary_cone_ineq(full_dim_projection(verts), subdiv, options);
 
-   perl::Object res(perl::ObjectType::construct<Scalar>("Cone"));
-   res.take("INEQUALITIES") << mats.first;
-   res.take("EQUATIONS") << mats.second;
+   BigObject res("Cone", mlist<Scalar>(),
+                 "INEQUALITIES", mats.first,
+                 "EQUATIONS", mats.second);
 
-   const Vector<Scalar> w=res.give("REL_INT_POINT");
-   const Vector<Scalar> slack= mats.first*w;
-
-   for(typename Entire<Vector<Scalar> >::const_iterator it=entire(slack); !it.at_end();++it){
-      if(*it==0)
-         return std::pair<bool,Vector<Scalar> >(false,Vector<Scalar>());
+   Vector<Scalar> w;
+   try {
+      const Vector<Scalar> wt = res.give("REL_INT_POINT");
+      w = wt;
+   } catch (const Undefined& e) {
+      // there is no relative internal point. If the trivial subdivision is asked for, we are ok; else the subdivision is not regular
+      if (subdiv.size() == 1 &&
+          subdiv[0] == sequence(0, verts.rows())) {
+         w = zero_vector<Scalar>(mats.first.cols());
+      } else return std::pair<bool,Vector<Scalar>>(false,Vector<Scalar>());
    }
-   return std::pair<bool,Vector<Scalar> >(true,w);
-}
+   const Vector<Scalar> slack = mats.first*w;
 
+   for (auto it=entire(slack); !it.at_end(); ++it) {
+      if (*it==0)
+         return std::pair<bool,Vector<Scalar>>(false,Vector<Scalar>());
+   }
+   return std::pair<bool,Vector<Scalar>>(true,w);
+}
 
 FunctionTemplate4perl("secondary_cone_ineq<Scalar>(Matrix<Scalar> Array<Set>; {equations => undef, lift_to_zero=>undef, lift_face_to_zero => undef, test_regularity=>0})");
 
+FunctionTemplate4perl("full_dim_projection<Scalar>(Matrix<Scalar>)");
 
 UserFunctionTemplate4perl("# @category Triangulations, subdivisions and volume"
                           "# For a given subdivision //subdiv// of //points// tests"
@@ -118,3 +127,10 @@ UserFunctionTemplate4perl("# @category Triangulations, subdivisions and volume"
                           "regularity_lp<Scalar>(Matrix<Scalar>,$;{equations => undef, lift_to_zero=>undef, lift_face_to_zero => 0, epsilon => 1/100})");
 
 } }
+
+
+// Local Variables:
+// mode:C++
+// c-basic-offset:3
+// indent-tabs-mode:nil
+// End:
